import { AllLayoutAlgorithms, LayoutFactory } from '@graphpolaris/graph-layout';
import {
  createReactFlowElements,
  expandSchema,
} from '@graphpolaris/schema/usecases';
import {
  useSchema,
  useSchemaLayout,
} from '@graphpolaris/shared/data-access/store';
import { MultiGraph } from 'graphology';
// import { AllLayoutAlgorithms, LayoutFactory } from '@graphpolaris/graph-layout';
import { useEffect, useMemo, useState } from 'react';
import ReactFlow, {
  ControlButton,
  Controls,
  FlowElement,
  ReactFlowProvider,
} from 'react-flow-renderer';
import styles from './schema.module.scss';

import {
  EntityRFPill,
  RelationRFPill,
  AttributeRFPill,
  ConnectionDragLine,
  ConnectionLine
} from '@graphpolaris/shared/ui/pills';
// import ConnectionDragLine from '@graphpolaris/shared/ui/pills';
// import AttributeRFPill from '@graphpolaris/shared/ui/pills';
// import EntityRFPill from '@graphpolaris/shared/ui/pills';
// import RelationRFPill from '@graphpolaris/shared/ui/pills';

interface Props {
  // content: string;
}

const onLoad = (reactFlowInstance: any) => {
  setTimeout(() => reactFlowInstance.fitView(), 100);
};

const nodeTypes = {
  entity: EntityRFPill,
  relation: RelationRFPill,
  attribute: AttributeRFPill,
};
const edgeTypes = {
  connection: ConnectionLine,
};

const Schema = (props: Props) => {
  const [elements, setElements] = useState([] as FlowElement[]);
  // In case the schema is updated
  const dbschema = useSchema();
  // const [dbschema, setSchema] = useState(useSchema());
  const [schemaLayout, setSchemaLayout] = useState(useSchemaLayout());

  // useEffect(() => {
  //   console.log('dbSchema', dbschema, dbschema.order);
  // }, [dbschema]);

  const expandedSchema = useMemo(() => expandSchema(dbschema), [dbschema]);

  useEffect(() => {
    if (dbschema == undefined || dbschema.order == 0) {
      return;
    }

    const layoutFactory = new LayoutFactory();
    console.log('schema Layout', schemaLayout, 'order', expandedSchema);
    const layout = layoutFactory.createLayout(
      schemaLayout as AllLayoutAlgorithms
    );
    layout?.layout(expandedSchema);

    const flowElements = createReactFlowElements(expandedSchema);
    setElements(flowElements);
    console.log(
      'update schema useEffect',
      dbschema,
      dbschema.order,
      flowElements
    );
  }, [dbschema, schemaLayout]);

  const graphStyles = { width: '100%', height: '500px' };

  return (
    <div
      style={{
        width: '100%',
        height: '100%',
      }}
    >
      {elements.length == 0 && <p>DEBUG: No Elements</p>}
      <ReactFlowProvider>
        <ReactFlow
          className={styles.schemaPanel}
          onlyRenderVisibleElements={false}
          nodesDraggable={false}
          nodeTypes={nodeTypes}
          edgeTypes={edgeTypes}
          connectionLineComponent={ConnectionDragLine}
          elements={elements}
          style={graphStyles}
          onLoad={onLoad}
        >
          <Controls
            showInteractive={false}
            showZoom={false}
            showFitView={true}
            className={styles.controls}
          >
            <ControlButton
              className={styles.exportButton}
              title={'Export graph schema'}
              onClick={(event) => {
                event.stopPropagation();
                // this.setState({
                //   ...this.state,
                //   exportMenuAnchor: event.currentTarget,
                // });
              }}
            >
              {/* <img src={exportIcon} width={21}></img> */}
            </ControlButton>
          </Controls>
        </ReactFlow>
      </ReactFlowProvider>
    </div>
  );
};

export default Schema;

// Fix layout of the schema
// create reactflow elements on xy coords
// connect reactflow elements together

// maybe ook gelijk instellingen knoppie fixen op alle panels
