import { SchemaFromBackend, Node, Edge } from '@graphpolaris/models';
import Graph, { MultiGraph } from 'graphology';

export class SchemaUtils {
  public static ParseSchemaFromBackend(
    schemaFromBackend: SchemaFromBackend
  ): Graph {
    const { nodes, edges } = schemaFromBackend;
    // Instantiate a directed graph that allows self loops and parallel edges
    const schemaGraph = new MultiGraph({ allowSelfLoops: true });
    // console.log('parsing schema');
    // The graph schema needs a node for each node AND edge. These need then be connected

    nodes.forEach((node: Node) => {
      schemaGraph.addNode(node.name, {
        name: node.name,
        attributes: node.attributes,
        x: 0,
        y: 0,
      });
    });

    

    // The name of the edge will be name + from + to, since edge names are not unique
    edges.forEach((edge: Edge) => {
      const edgeID = [edge.name, '_', edge.from, edge.to].join(''); //ensure that all interpreted as string

      // This node is the actual edge
      schemaGraph.addDirectedEdgeWithKey(edgeID, edge.from, edge.to, {
        attribute: edge.attributes,
      });
    });
    return schemaGraph;
  }
}
