import { handles } from '@graphpolaris/querybuilder/usecases';
import React from 'react';
import { EdgeProps, getSmoothStepPath, Position } from 'react-flow-renderer';

/**
 * A custom query element edge line component.
 * @param {EdgeProps} param0 The coordinates for the start and end point, the id and the style.
 */
//  export const EntityRFPill = React.memo(({ data }: { data: any }) => {
export function ConnectionLine({
  id,
  sourceX,
  sourceY,
  targetX,
  targetY,
  style,
  sourceHandleId,
  targetHandleId,
}: EdgeProps) {
  //Centering the line
  sourceY -= 3;
  targetY -= 3;

  // Correct line positions with hardcoded numbers, because react flow lacks this functionality
  // if (sourceHandleId == ) sourceX += 2;

  // if (targetHandleId == Handles.ToAttributeHandle) targetX += 2;

  let spos: Position = Position.Bottom;
  if (sourceHandleId == handles.relation.fromEntity) {
    spos = Position.Left;
    sourceX += 7;
    sourceY += 3;
  } else if (sourceHandleId == handles.relation.toEntity) {
    spos = Position.Right;
    sourceX -= 2;
    sourceY -= 3;
  } else if (
    sourceHandleId !== undefined &&
    sourceHandleId !== null &&
    sourceHandleId.includes('functionHandle')
  ) {
    spos = Position.Top;
    sourceX -= 4;
    sourceY += 3;
  }

  let tpos: Position = Position.Bottom;
  if (targetHandleId == handles.relation.fromEntity) {
    tpos = Position.Left;
    targetX += 7;
    targetY += 3;
  } else if (targetHandleId == handles.relation.toEntity) {
    tpos = Position.Right;
    targetX -= 2;
    targetY -= 3;
  }

  // Create smoothstep line
  const path = getSmoothStepPath({
    sourceX: sourceX,
    sourceY: sourceY,
    sourcePosition: spos,
    targetX: targetX,
    targetY: targetY,
    targetPosition: tpos,
  });

  return (
    <g stroke="#2e2e2e">
      <path id={id} fill="none" strokeWidth={3} style={style} d={path} />
    </g>
  );
}
